﻿using System;
using System.Collections.Generic;
using System.Linq;

using Styx;

using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

namespace BgBot
{
    internal class BannerHelper
    {
        private static readonly bool IsHorde = StyxWoW.Me.IsHorde;

        public static List<WoWGameObject> GetNearbyBanners()
        {
            // Make sure we only get banners within range of us. Or we'll end up ignoring the entire BG just to grab banners. -_-
            return
                ObjectManager.ObjectList.OfType<WoWGameObject>().Where(go => go.DistanceSqr <= 40 * 40 && go.Model.Contains("BattlefieldBanners")).
                    ToList();
        }

        private static bool EnemyPlayersNearBanner(WoWGameObject bannerLocation)
        {
            return ObjectManager.GetObjectsOfType<WoWPlayer>(true,false).Any(p => p.IsHostile && p.Location.DistanceSqr(bannerLocation.Location) <= 10 * 10);
        }

        private static bool FriendlyPlayersCapturing(WoWGameObject go)
        {

                return
                    ObjectManager.GetObjectsOfType<WoWPlayer>(true, true)
                                 .Any(p => p.Location.DistanceSqr(go.Location) <= 10 * 10 && p.IsFriendly && p.IsCasting && p.CastingSpell.Name == "Capturing" && p != StyxWoW.Me);

        }

        private static bool BannerUsable(WoWGameObject o)
        {
            if (IsCaptured(o))
                return false;
            if (FriendlyPlayersCapturing(o))
                return false;
            if (EnemyPlayersNearBanner(o))
                return false;

            string m = o.Model.ToLower();
            if (IsHorde)
            {
                return m.Contains("alliance") || m.Contains("neutral");
            }
            return m.Contains("horde") || m.Contains("neutral");
        }

        public static WoWGameObject NearbyUsableBanner
        {
            get
            {
                return GetNearbyBanners().Where(BannerUsable).OrderBy(o => o.DistanceSqr).FirstOrDefault();
            }
        }

        private static bool IsCaptured(WoWGameObject o)
        {
            return o.Model.ToLower().Contains("bw");
        }
    }
}