﻿using System;
using System.Threading;

using Styx;
using Styx.Common;
using Styx.CommonBot;

using Styx.TreeSharp;
using Styx.WoWInternals;


using Action = Styx.TreeSharp.Action;

namespace BgBot.Behaviors
{
    public class ActionAcceptBattlefieldPort : Action
    {
        #region Overrides of Behavior

        /// <summary>
        /// Actually executes one behavior tick
        /// </summary>
        /// <returns></returns>
        protected override RunStatus Run(object context)
        {
            int delay = new Random().Next(5, 10);

            Logging.Write("Waiting {0} seconds before accepting port", delay);
            var end = DateTime.Now.AddSeconds(delay);
            while (DateTime.Now < end)
            {
                // If we somehow get in combat, just fail. Let it run back up the tree so we can exit on mixed mode.
                if (StyxWoW.Me.Combat || StyxWoW.Me.PetInCombat)
                    return RunStatus.Failure;
                Thread.Sleep(100);
            }

            Battlegrounds.AcceptBattlefieldPort(Battlegrounds.GetBGIndexWithStatus(BattlegroundStatus.Confirm), true);

            // Wait to leave game...
            int tickCount = Environment.TickCount;
            while (StyxWoW.IsInGame && Environment.TickCount - tickCount <= 120000)
            {
                // If we somehow get in combat, just fail. Let it run back up the tree so we can exit on mixed mode.
                if (StyxWoW.Me.Combat || StyxWoW.Me.PetInCombat)
                    return RunStatus.Failure;
                
                Thread.Sleep(100);
            }

            // Wait to enter game!
            tickCount = Environment.TickCount;
            while (!StyxWoW.IsInGame && Environment.TickCount - tickCount <= 120000)
                Thread.Sleep(100);

            if (!StyxWoW.IsInGame)
                return RunStatus.Failure;

            ObjectManager.Update();
            Logging.Write("About to fire FireBattlegroundEntered");
            BotEvents.Battleground.FireBattlegroundEntered();
            Logging.Write("About to return out of ActionAcceptBattlefieldPort");
            return RunStatus.Success;
        }

        #endregion
    }
}