﻿using System;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Threading;

using Bots.Grind;
using Styx.Common;
using Styx.Helpers;

using Styx.WoWInternals;

namespace BgBot
{
    partial class BgBot
    {
        private void HandleOnStart(EventArgs args)
        {
            if (Battlegrounds.IsInsideBattleground)
            {
                // Set the targeting filters
                HandleBgEntered(BattlegroundType.None);
            }
            else
            {
                HandleBgLeft(null);
            }
        }

        private void HandleBgLeft(EventArgs args)
        {
            // Disposing the battleground will remove targeting filters, and any other crap we do.
            // This keeps it generic, so we don't have to worry about it.
            if (_currentBattleground != null)
            {
                _currentBattleground.Dispose();
            }

            _currentBattleground = null;
            _currentProfile = null;
            _currentHotspot = null;

            Log("HandleBgLeft");


            RemoveTargetingFilters();
            LevelBot.SetDefaultQueryFilter();
        }

        private void HandleBgEntered(BattlegroundType type)
        {
            if (_currentBattleground != null)
            {
                Log("Battleground object not disposed, cleaning it now");
                _currentBattleground.Dispose();
                _currentBattleground = null;
            }

            _currentBattleground = GetCurrentBattleground();

            if (_currentBattleground != null)
            {
                _currentProfile =
                    new PvpProfile(
                        string.Format(
                            "{0}\\Bots\\BGBot\\Profiles\\{1}.xml", Styx.Common.Utilities.AssemblyDirectory, _currentBattleground.ProfileName));
                PvpProfile.Current = _currentProfile;

                SetTargetingFilters();
                SetPvpQueryFilter();
            }
        }
    }
}