﻿using System;
using System.Collections.Generic;
using System.Linq;

using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.Helpers;


using Styx.WoWInternals;
using Styx.WoWInternals.World;
using Styx.WoWInternals.WoWObjects;

namespace BgBot
{
    partial class BgBot
    {
        private bool _AddedFilters;

        public void SetTargetingFilters()
        {
            //Logging.Write("Zomg added filters " + new Random().Next());
            if (!_AddedFilters)
            {
                Targeting.Instance.IncludeTargetsFilter += IncludeTargetsFilter;
                Targeting.Instance.RemoveTargetsFilter += RemoveTargetsFilter;
                _AddedFilters = true;
            }
        }

        public void RemoveTargetingFilters()
        {
            Targeting.Instance.IncludeTargetsFilter -= IncludeTargetsFilter;
            Targeting.Instance.RemoveTargetsFilter -= RemoveTargetsFilter;
            _AddedFilters = false;
        }

        private static void RemoveTargetsFilter(List<WoWObject> units)
        {
            var includeMountedPlayers = BBRSettings.Instance.IncludeMountedPlayers;
            units.RemoveAll(u =>
                                {
                                    var player = u as WoWPlayer;

                                    if (player == null)
                                        return true;//Was false...changed to true, filter out anything thats not a player...

                                    if (!includeMountedPlayers && player.Mounted)
                                        return true;

                                    if (player.Shapeshift == ShapeshiftForm.SpiritOfRedemption)
                                        return true;

                                    if (player.IsDead)
                                        return true;

                                    return false;
                                });
        }

        public static void IncludeTargetsFilter(List<WoWObject> incomingUnits, HashSet<WoWObject> outgoingUnits)
        {
            bool isMounted = StyxWoW.Me.Mounted;
            bool withinRange = StyxWoW.Me.Location.DistanceSqr(Instance.CurrentHotspot.Position) <= 50*50;

            // Ignore ALL players out of range while we're mounted!
            if (isMounted && !withinRange)
            {
                return;
            }

            bool isHorde = StyxWoW.Me.IsHorde;

            bool includeMountedPlayers = BBRSettings.Instance.IncludeMountedPlayers;

            var toBeAdded = new List<WoWUnit>();
            foreach (WoWObject obj in incomingUnits)
            {
                if (!(obj is WoWPlayer))
                {
                    continue; // The default include targets filter includes units in combat with us, so we don't need to check for that here.
                }

                WoWPlayer p = obj.ToPlayer();

                if (p.Mounted && !includeMountedPlayers)
                    continue;

                if (p.IsDead)
                    continue;

                if (isHorde == !p.IsHorde)
                {
                    toBeAdded.Add(p);
                }
            }

            int count = toBeAdded.Count > 5 ? 5 : toBeAdded.Count;
            var lines = new WorldLine[count];
            WoWPoint myLoc = StyxWoW.Me.GetTraceLinePos();

            for (int i = 0; i < count; i++)
            {
                lines[i] = new WorldLine(myLoc, toBeAdded[i].GetTraceLinePos());
            }

            if (lines.Length != 0)
            {
                bool[] results;
                GameWorld.MassTraceLine(lines, Styx.WoWInternals.World.TraceLineHitFlags.LineOfSight, out results);

                for (int i = 0; i < count; i++)
                {
                    if (!results[i])
                    {
                        outgoingUnits.Add(toBeAdded[i]);
                    }
                }
            }
        }
    }
}