﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Threading;
using System.Windows.Forms;
using System.Windows.Media;
using BgBot.Behaviors;
using BgBot.Gui;
using BgBot.Logic;

using Bots.Grind;

using CommonBehaviors.Actions;
using CommonBehaviors.Decorators;

using Levelbot.Actions.Combat;
using Levelbot.Actions.Death;

using Styx;
using Styx.Common;
using Styx.Common.Helpers;
using Styx.CommonBot;
using Styx.CommonBot.POI;
using Styx.CommonBot.Profiles;
using Styx.CommonBot.Routines;
using Styx.CommonBot.Coroutines;
using Styx.Helpers;

using Styx.Pathing;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.World;
using Styx.WoWInternals.WoWObjects;
using MathEx = Tripper.Tools.Math.MathEx;
using Vector2 = Tripper.Tools.Math.Vector2;

//using Action = .Action;
using Action = Styx.TreeSharp.Action;
namespace BgBot
{
    public partial class BgBot : BotBase
    {
        // Less typing.
        private static LocalPlayer Me { get { return StyxWoW.Me; } }

        public override bool IsPrimaryType
        {
            get { return false; }
        }

        public override bool RequirementsMet
        {
            get
            {
                // Obviously. Any time we're in a BG, we should be running the BG bot. Period.
                if (Battlegrounds.IsInsideBattleground)
                    return true;

                // If we're in combat, ignore this entire bot. (Only outside of BGs)
                // Above check makes sure this doesn't get hit while in BGs. So it should be safe to do this!
                // This stops us from dying stupidly.
                if (Me.Combat)
                    return false;

                // Obviously if we are dead we can't run this bot, not even queue or accept bg ports.
                if (!Me.IsAlive)
                    return false;

                // Ignore if we have deserter for some reason
                if (StyxWoW.Me.HasAura("Deserter"))
                    return false;

                // Check for stuff we need to queue for
                int queueCount = 0;
                if (BBRSettings.Instance.BG1 != BattlegroundType.None)
                    queueCount++;
                if (BBRSettings.Instance.BG2 != BattlegroundType.None)
                    queueCount++;
                var queuedBgs = Battlegrounds.BattlegroundStatuses.Count(s => s == BattlegroundStatus.Queued);
                // Make sure we can queue up.
                if (queuedBgs < queueCount)
                    return true;

                // And lastly... if we have a queue that popped, make sure we can get in it!
                return !Battlegrounds.IsInsideBattleground && Battlegrounds.GetBGIndexWithStatus(BattlegroundStatus.Confirm) != -1;
            }
        }

        private readonly WaitTimer _hotspotCheckTimer = new WaitTimer(TimeSpan.FromSeconds(8));//Was 15, lets try 5

        private readonly Stopwatch _ressTimer = new Stopwatch();

        private Battleground _currentBattleground;

        private PvpHotspot _currentHotspot;

        private PvpProfile _currentProfile;

        private Composite _root;

        public Battleground CurrentBg { get { return _currentBattleground ?? (_currentBattleground = GetCurrentBattleground()); } }

        public PvpProfile CurrentProfile { get { return _currentProfile ?? (_currentProfile = GetCurrentProfile()); } }

        public static Form _form;

        public override Form ConfigurationForm
        {

            get { return _form ?? (_form = new FormBGSettings()); }
        }

        public override string Name { get { return "BG Bot"; } }

        internal static BgBot Instance { get; set; }

        public override Composite Root
        {
            get
            {
                return _root ?? (_root =
                        new PrioritySelector(
                            CreateOutsideBattlegroundBehavior(),
                            CreateInsideBattlegroundBehavior(),
                            new ActionIdle())
                            );
            }
        }

        public override PulseFlags PulseFlags { get { return PulseFlags.All; } }
        private bool _updatingHotspot;
        public PvpHotspot CurrentHotspot
        {
            get
            {
                if (_hotspotCheckTimer.IsFinished && !_updatingHotspot)
                {
                    // Ensure no recursion.
                    _updatingHotspot = true;
                    try
                    {
                        List<PvpHotspot> hotspots = _currentProfile.FactionSpecificHotspots;



                        /*foreach(ulong guid in Me.GroupInfo.RaidMemberGuids)
                        {
                            WoWPlayer @object = ObjectManager.GetObjectByGuid<WoWPlayer>(guid);
                            if(@object != null)
                            {
                                WoWPoint point = @object.Location;
                                if (@object.IsDead || @object.IsGhost)
                                    continue;
                                    
                                pos.Add(new Vector2(point.X, point.Y));
                            }
                            else
                            {
                                //WoWPartyMember.PartyMemberInfo info = WoWPartyMember.GetInfo(guid);
                                var info = Me.GroupInfo.RaidMembers.FirstOrDefault(r => r.Guid == guid);

                                pos.Add(new Vector2(info.Location.X, info.Location.Y));
                            }
                        }*/






                        //Rewrite:
                        //IEnumerable<Vector2> pos = Me.RaidMembers.Where(r => r.IsAlive).Select(player => new Vector2(player.X, player.Y));
                        //This was not getting everyone cause it couldn't convert units that were far away into player objects i think
                        //This should work better?
                        IEnumerable<Vector2> pos = (from groupMember in StyxWoW.Me.GroupInfo.RaidMembers
                                                    let player = groupMember.ToPlayer()
                                                    let isAlive = player != null ? player.IsAlive : !groupMember.Dead && !groupMember.Ghost
                                                    where isAlive
                                                    let location = player != null ? player.Location : groupMember.Location3D
                                                    select new Vector2(location.X, location.Y));

                        _currentHotspot = _currentBattleground.GetFightLocation(pos, hotspots);
                        BgBot.Log("Best: " + _currentHotspot);
                        _hotspotCheckTimer.Reset();
                    }
                    finally
                    {
                        _updatingHotspot = false;
                    }
                }
                return _currentHotspot;
            }
        }

        private PvpProfile GetCurrentProfile()
        {
            if (CurrentBg != null)
            {
                return new PvpProfile(
                    string.Format(
                        "{0}\\Bots\\BGBot\\Profiles\\{1}.xml", Styx.Common.Utilities.AssemblyDirectory, _currentBattleground.ProfileName));
            }
            else
            {
                //Logging.Write("Current profile was null" + _currentBattleground.ProfileName);
                BgBot.Log("Current battleground is null!");
            }


            return null;
        }

        public override void Initialize()
        {

            BBRSettings.Instance.Load();

            Log("Current version is:" + Svn.RevString);

            TreeHooks.Instance.ReplaceHook("BgBot_Inside_1", new PrioritySelector(BGInternals));


            base.Initialize();
        }

        public override void Start()
        {
            Instance = this;
            ProfileManager.LoadEmpty();
            BotEvents.Battleground.OnBattlegroundEntered += HandleBgEntered;
            BotEvents.Battleground.OnBattlegroundLeft += HandleBgLeft;
            HandleOnStart(null);
        }

        public override void Stop()
        {
            BotEvents.Battleground.OnBattlegroundEntered -= HandleBgEntered;
            BotEvents.Battleground.OnBattlegroundLeft -= HandleBgLeft;
            // Remove the target filters. Kthx.
            RemoveTargetingFilters();
            Instance = null;

            // Load the profile we had before.
            string fullPath = CharacterSettings.Instance.LastUsedPath;

            if (!File.Exists(fullPath))
            {
                if (!string.IsNullOrEmpty(fullPath))
                    BgBot.Log("Honorbuddy was unable to find file {0}", fullPath);

                return;
            }

            ProfileManager.LoadNew(fullPath);
        }

        private static float CalculatePathPrecision()
        {
            float curSpeed = Me.MovementInfo.ForwardSpeed;
            float pathPrecision = curSpeed * 0.10f;

            return MathEx.Clamp(pathPrecision, 1.0f, 10f);
        }

        public override void Pulse()
        {
            if (!Battlegrounds.IsInsideBattleground)
                return;

            if (_currentBattleground != null && _currentBattleground.BgType != Battlegrounds.Current)
            {
                HandleOnStart(null);
            }

            Navigator.PathPrecision = CalculatePathPrecision();
        }

        /// <summary>
        ///   Gets the current battleground. This returns a NEW INSTANCE of a <see cref = "Battleground" /> object. This is only to be called when first entering a BG.
        /// </summary>
        /// <remarks>
        ///   Created 12/22/2010.
        /// </remarks>
        /// <returns>The current battleground.</returns>
        public Battleground GetCurrentBattleground()
        {
            switch (Battlegrounds.Current)
            {
                case BattlegroundType.AV:
                    return new AlteracValley();
                case BattlegroundType.WSG:
                    return new WarsongGulch();
                case BattlegroundType.AB:
                    return new ArathiBasin();
                case BattlegroundType.EotS:
                    return new EyeOfTheStorm();
                case BattlegroundType.SotA:
                    return new StrandOfTheAncients();
                case BattlegroundType.IoC:
                    return new IsleOfConquest();
                case BattlegroundType.BfG:
                    return new BattleForGilneas();
                case BattlegroundType.TP:
                    return new TwinPeaks();
                case BattlegroundType.ToK:
                    return new TempleOfKotmogu();
                case BattlegroundType.SM:
                    return new SilvershardMines();
                case BattlegroundType.DG:
                    return new DeepwindGorge();
                default:
                    return null;
            }
        }

        #region Outside Battleground Behavior

        public Composite CreateOutsideBattlegroundBehavior()
        {
            return new PrioritySelector(
                // Are we outside of the bg?
                new Decorator(ret => !Battlegrounds.IsInsideBattleground,
                    new PrioritySelector(
                        LevelBot.CreateDeathBehavior(),
                        LevelBot.CreateVendorBehavior(),
                // Do we need to wait for 'Deserter' to expire?
                        new Decorator(
                            ret => Me.HasAura("Deserter"),
                            new Sequence(new ActionSetActivity("Waiting for deserter to expire."), new ActionIdle())),
                        new Decorator(
                            ret =>
                            !Battlegrounds.IsInsideBattleground &&
                            Battlegrounds.GetBGIndexWithStatus(BattlegroundStatus.Confirm) != -1,
                            new Sequence(
                                new ActionSetActivity("Accepting battlefield port..."),
                                new ActionAcceptBattlefieldPort())),
                        new DecoratorNeedToEnqueue(
                            new Sequence(
                                new ActionSetActivity("Queueing for a battleground"),
                                new ActionEnqueueForBattlegrounds())),

                        //new Decorator(
                //    ret => WaitedTooLongInQueue(),
                //    new Sequence(
                //        new ActionSetActivity("Requeueing battleground!"),
                //        new Action(ret => UnqueueOverTimeBgs()))),
                // Do we need to wait for the port?
                        new Decorator(
                            ret => Battlegrounds.GetBGIndexWithStatus(BattlegroundStatus.Queued) != -1,
                            new Sequence(
                                new ActionSetActivity("Waiting for battlefield port"),
                                new ActionIdle()))


                        )));
        }
        #endregion

        #region Inside Battleground Behavior

        public Composite CreateInsideBattlegroundBehavior()
        {
            return
                new Decorator(
                    ret => Battlegrounds.IsInsideBattleground && CurrentProfile != null && CurrentBg != null,
                    new HookExecutor("BgBot_Inside_1", "Inside battleground logic.")

                    //new PrioritySelector(BGInternals)
                );
        }

        public Composite[] BGInternals
        {
            get
            {
                return
                        new[]{//new ActionLogger("CreateInsideBattlegroundBehavior"),
                        //new FailLogger("Top of the inside tree"),
                        new HookExecutor("BgBot_Special_1", "Top of the inside battleground tree."),


                        // Leave if the bg is finished.
                        new Decorator(ret => Battlegrounds.Finished, new ActionLeaveBattleground()),

                        // Do we need to wait for the bg to start? SotA excluded.
                        new Decorator(ret =>  Me.HasAura("Preparation"),
                            new PrioritySelector(
                                
                                //Idle on the boat so we don't run around.
                                new Decorator(r=> Battlegrounds.Current == BattlegroundType.SotA && Me.IsOnTransport, new Wait(10,new ActionIdle())),
                                // Buffs and shit.
                                new Decorator(cnd => RoutineManager.Current != null && RoutineManager.Current.PreCombatBuffBehavior != null,RoutineManager.Current.PreCombatBuffBehavior),
                                
                                // Resting eat/drink
                                new Decorator(cnd => RoutineManager.Current != null && RoutineManager.Current.RestBehavior != null,RoutineManager.Current.RestBehavior),
                                    
                                // Mount if we can.
                                new Decorator(ret => Battlegrounds.Current != BattlegroundType.EotS && !Me.Mounted && Mount.CanMount(),
                                    new Action(ret => Mount.StateMount(GetWaitingSpot))
                                    ),

                                new Action(ret => Navigator.MoveTo(GetWaitingSpot()))
                                )
                            ),

                        // Death behavior here...
                        CreatePvpDeathBehavior(),

                        // Clear the target if its far away. Don't get kited or if it is a pet
                        new Decorator(ret=>Me.CurrentTarget != null && (Me.CurrentTarget.Distance > 60 ||  Me.CurrentTarget.IsPet || Me.CurrentTarget.Shapeshift == ShapeshiftForm.SpiritOfRedemption),new Action(ret=>Me.ClearTarget())),

                        //If we don't have a target and we are incombat alot of CC's dont do anything about that.
                        new DecoratorNeedToFindPVPTarget(ret => Me.Combat && !Me.GotTarget, TargetNewUnit),

                        //new FailLogger("Combat Logic"),
                        // Combat behavior here...
                        //LevelBot.CreateCombatBehavior(),

                        new Decorator(r=>Me.GotTarget,new PrioritySelector(r=>Me.CurrentTarget,
                            new Decorator(r=> !Me.Combat,new PrioritySelector(
                              new Decorator(r=> !Me.Mounted, RoutineManager.Current.PreCombatBuffBehavior),
                              new Decorator(r => ((WoWUnit)r).Distance <= Targeting.PullDistance && ((WoWUnit)r).InLineOfSight, RoutineManager.Current.PullBehavior),
                              new Decorator(r => ((WoWUnit)r).Distance > Targeting.PullDistance || !((WoWUnit)r).InLineOfSight, new ActionMoveToTarget())
                                
                                
                               )),
                            new Decorator(r => Me.Combat, new PrioritySelector(


                                new Decorator(r=>((WoWUnit)r).IsDead,new Action(r=>Me.ClearTarget())),
                                RoutineManager.Current.HealBehavior,
                                RoutineManager.Current.CombatBuffBehavior,
                                RoutineManager.Current.CombatBehavior))
                            
                            
                            )),
                        
                        // Banner grabbing
                        BannerLogic,

                        // SotA behavior here.. getting off the boat etc.
                        //CreateSotABehavior(),

                        CreateMoveAndTarget()};
            }
        }

        private Composite BannerLogic
        {
            get
            {
                return new Decorator(ret =>!Me.Combat && (Battlegrounds.Current == BattlegroundType.AB || Battlegrounds.Current == BattlegroundType.BfG) && BannerHelper.NearbyUsableBanner != null,
                    new PrioritySelector(
                    // There's a banner to capture nearby, grab it!
                        new Decorator(
                            ret => Me.Location.Distance(BannerHelper.NearbyUsableBanner.Location) > 4f,
                            new Action(
                                ret =>
                                {
                                    TreeRoot.StatusText = "Moving to capture " + BannerHelper.NearbyUsableBanner.Name + " banner.";
                                    Navigator.MoveTo(BannerHelper.NearbyUsableBanner.Location);
                                })),

                        // If we're not casting yet, fucking grab the banner!
                        new Decorator(ret => !Me.IsCasting,
                                      new Sequence(
                                          new DecoratorContinue(ret => Me.IsMoving, new Action(ret => WoWMovement.MoveStop())),

                                          new Action(ret =>
                                          {
                                              Styx.CommonBot.Coroutines.CommonCoroutines.SleepForLagDuration();
                                              TreeRoot.StatusText = "Capturing the " + BannerHelper.NearbyUsableBanner.Name + " banner.";
                                              BannerHelper.NearbyUsableBanner.Interact();
                                          }))),

                        // Wait to finish casting!
                        new Wait(15, ret => !Me.IsCasting || Me.Combat, new ActionIdle())));
            }
        }

        #region Get Gate Spot

        public static readonly WaitTimer _gateTimer = new WaitTimer(new TimeSpan(0, 2, 20));//Was 30, reduced to 20 as it idles a bit after the start, this should help fix it.
        private WoWPoint _gatespot;

        private WoWPoint GetWaitingSpot()
        {
            if (!_gateTimer.IsFinished)
                return _gatespot;

            WoWPoint spot = Me.Location;
            LocalPlayer me = Me;

            switch (Battlegrounds.Current)
            {
                case BattlegroundType.WSG:
                    spot = me.IsHorde
                               ? new WoWPoint(939.3743f, 1427.106f, 345.5172f)
                               : new WoWPoint(1521.935f, 1484.45f, 351.9924f);
                    break;

                case BattlegroundType.AB:
                    spot = me.IsHorde
                               ? new WoWPoint(702.9686f, 703.5789f, -16.16586f)
                               : new WoWPoint(1290.146f, 1286.864f, -14.30505f);
                    break;

                case BattlegroundType.AV:
                    spot = me.IsHorde
                               ? new WoWPoint(-1393.961, -556.4873, 55.24042)
                               : new WoWPoint(804.7042, -494.523, 99.9165);
                    break;

                case BattlegroundType.BfG:
                    spot = me.IsHorde
                               ? new WoWPoint(1402.404f, 977.3433f, 7.389421f)
                               : new WoWPoint(910.358f, 1338.934f, 27.60992f);
                    break;

                case BattlegroundType.TP:
                    spot = me.IsHorde
                               ? new WoWPoint(1557.268f, 325.4247f, 1.533998f)
                               : new WoWPoint(1402.154f, 977.9141f, 7.346243f);
                    break;


                case BattlegroundType.ToK:
                    spot = me.IsHorde
                              ? new WoWPoint(1780.424f, 1598.724f, 33.78216f)
                              : new WoWPoint(1784.386f, 1087.162f, 26.17026f);
                    break;


                case BattlegroundType.SM:
                    spot = me.IsHorde
                              ? new WoWPoint(630.1702f, 230.1736, 328.8382f)
                              : new WoWPoint(850.605f, 135.8336f, 328.8382f);
                    break;
                case BattlegroundType.DG:
                    //<Vendor Name="horde" Entry="0" Type="Repair" X="-88.53345" Y="806.4467" Z="133.2523" />
                    //<Vendor Name="alliance" Entry="0" Type="Repair" X="-238.4184" Y="191.8389" Z="133.4921" />
                    spot = me.IsHorde ? new WoWPoint(-88.53345,806.4467,133.2523) : new WoWPoint(-238.4184,191.8389,133.4921);
                    break;


                // Don't move during EOTS. Period.
                case BattlegroundType.EotS:
                    _gatespot = Me.Location;
                    return Me.Location;
            }

            var rand = new Random();
            _gatespot = spot.RayCast(WoWMathHelper.DegreesToRadians(rand.Next(1, 360)), rand.Next(3, 4));

            spot.Z += 2.132f;
            _gatespot.Z += 2.132f;

            if (!GameWorld.IsInLineOfSight(spot, _gatespot))
            {
                GetWaitingSpot();
            }

            _gatespot.Z -= 2.132f;

            _gateTimer.Reset();
            return _gatespot;
        }

        #endregion

        #endregion

        #region Death Behavior

        public Composite CreatePvpDeathBehavior()
        {
            return new PrioritySelector(

                // Release from corpse if we are dead.
                new Decorator(ret => Me.IsDead, new ActionReleaseFromCorpse()),

                // Are we a ghost?
                new Decorator(ret => Me.IsGhost,

                    // Set the context to the spirit healer.
                    new PrioritySelector(ret => ObjectManager.GetObjectsOfType<WoWUnit>().Where(unit => (unit.IsSpiritHealer || unit.IsSpiritGuide) && unit.Distance < 20).OrderBy(r => r.Distance).FirstOrDefault(),

                        // Start the ress timer if it isn't running.
                        new Decorator(ret => !_ressTimer.IsRunning,
                           new Sequence(
                               new ActionLogger("Restimer is not running, starting it"),
                               new Action(ret => _ressTimer.Start()))),

                        // Reset the timer if it's expired.
                        new Decorator(
                            ret => _ressTimer.Elapsed.TotalSeconds >= 70,
                            new PrioritySelector(
                                new FailLogger("Restimer expired, resetting it"),
                                new Action(ret => _ressTimer.Reset()))),



                        // Stop moving if we are moving if the ressTimer hasn't expired.
                        new Decorator(
                            ret => ((ret != null && ((WoWUnit)ret).Distance < 10) || ret == null) && Me.IsMoving && _ressTimer.Elapsed.TotalSeconds < 64,
                            new PrioritySelector(
                                new FailLogger("Stop moving you biatch! we are waiting for a mass ress"),
                                new Action(ret => WoWMovement.MoveStop()))),

                        // Move to the spirit guide if it's too far away.
                /*new Decorator(
                    ret => ret != null && _ressTimer.Elapsed.TotalSeconds < 64 && ((WoWUnit)ret).Distance > 15,
                    new PrioritySelector(
                        new FailLogger("Moving to spirit guide! we are > 15 yards from it"),
                        new Action(ret => Navigator.MoveTo(((WoWUnit)ret).Location)))),*/

                        // If 90 seconds has passed and we are still a ghost let's interact with the spirit guide.
                        new Decorator(
                            ret => ret != null && _ressTimer.Elapsed.TotalSeconds >= 64,
                            new PrioritySelector(
                // Move to the spirit guide.
                                new Decorator(
                                    ret => !((WoWUnit)ret).WithinInteractRange,
                                    new PrioritySelector(
                                        new FailLogger("Moving to spirit guide"),
                                        new Action(ret => Navigator.MoveTo(((WoWUnit)ret).Location)))),

                                // Make sure we aren't moving.
                                new Decorator(
                                    ret => Me.IsMoving,
                                    new Sequence(
                                        new ActionLogger("Making sure we aren't moving"),
                                        new Action(ret => WoWMovement.MoveStop()),
                // Wait some
                                        new ActionLogger("Waiting 2 seconds"),
                                        new Action(r => Thread.Sleep(2000)))),
                //new ActionSleep(2000))),
                                new Decorator(
                                    ret => ((WoWUnit)ret).WithinInteractRange,
                                    new Sequence(
                // Interact with the spirit guide.
                                        new ActionLogger("Interacting with spirit guide"),
                                        new Action(ret => ((WoWUnit)ret).Interact()),
                // Reset the timer as we are done.
                                        new ActionLogger("Resetting restimer"),
                                        new Action(ret => _ressTimer.Reset()))))),

                        new ActionAlwaysSucceed())));
        }

        #endregion

        #region MoveTo & Target Behaviors

        private Composite TargetNewUnit
        {
            get
            {
                //Do we need to find a new target?
                return
                    // Sequence for setings the current target and POI
                    new Sequence(
                    // Debug message
                        new ActionLogger("[RB] DNTFT -> S"),
                    // Set our target
                        new Action(ret => Targeting.Instance.FirstUnit.Target()),
                        new ActionLogger("[RB] SetTarget Finished. Waiting."),
                    // Wait until we got a target!
                        new Wait(3, ret => Me.GotTarget, new ActionIdle()),
                    //Was 5, reduce to 3, try to increase responsiveness.
                        new ActionLogger("[RB] Finished waiting, we got a target."));
                // Finally set the POI type to Kill!
                //new ActionSetPoi(ret => new BotPoi(Me.CurrentTarget, PoiType.Kill)));
            }
        }

        private Composite CreateMoveAndTarget()
        {
            return new PrioritySelector(

                new DecoratorNeedToFindPVPTarget(ret => !Me.GotTarget, TargetNewUnit),
                //new DecoratorNeedToFindPVPTarget(new DecoratorIsNotPoiType(PoiType.Kill, TargetNewUnit)),
                //new FailLogger("[RB] MoveToTarget"),

                new PrioritySelector(
                //new Decorator(NeedToMoveToTarget, new ActionMoveToTarget()),
                // Clear the poi if the POI type is kill and... 
                // we don't have a target and the first unit in the targetlist isn't null and the distance is less than the pulldistance
                // or, if we have a target and our currenttargetguid doesn't equal the guid of the first unit in the targetlist.
                //new Decorator(NeedToClearPoi, new ActionClearPoi("NeedToClearPOI is true. [No Suitable Target]")),
                //new DecoratorIsNotPoiType(PoiType.Kill,
                // Do we need to move to a hotspot?

                new Decorator(r => NeedToStopMovement, new Action(r =>
                                                                      {
                                                                          Logging.WriteDiagnostic("Within " + CurrentHotspot.Position.Distance(Me.Location) + "yds of " + CurrentHotspot.Name + " stopping movement");
                                                                          Navigator.PlayerMover.MoveStop();
                                                                      })),
                        new Decorator(
                            NeedToMoveToHotspot,
                // DO EET !
                            new Action(
                                delegate
                                {
                                    Logging.WriteDiagnostic("Moving to " + CurrentHotspot.Name);
                                    // Since the current ShouldMount always returns true in BGs/Instances, we do our own check here.
                                    // Really makes no sense to mount, os we can move 10yds
                                    if (CurrentHotspot.Position.Distance(Me.Location) > CharacterSettings.Instance.MountDistance && !Me.Mounted)
                                    {

                                        //Navigator.PlayerMover.MoveStop();
                                        Mount.StateMount(() => CurrentHotspot.Position);
                                        //Flightor.MountHelper.MountUp();
                                    }
                                    TreeRoot.StatusText = "Moving to " + CurrentHotspot.Name;
                                    Navigator.MoveTo(CurrentHotspot.Position);
                                }))));//);
        }

        private bool NeedToMoveToHotspot(object context)
        {
            // Only move to hotspots when its > 30yds away, and we're not in combat
            // Also make sure we're not mounted, as we could randomly grab aggro from shit on the way to the next hotspot.

            //Remove mounted check, im pretty sure this was copyed from somewhere else.
            //Lets try 15 for hotspots that are close together, or we might be waiting outside the bounds of one.
            //Logging.WriteDiagnostic("NeedToMoveToHotspot");

            return CurrentHotspot.Position.Distance(Me.Location) > CurrentHotspot.moveRadius && !Me.Combat;
        }

        private static string _last;

        public static void Log(string input) //use for slogging
        {
            if (input != _last)
            {
                _last = input;
                Logging.Write(Colors.LawnGreen, "[Bg Bot]{0}", input);
            }

        }

        public static void Log(string format, params object[] args) //use for slogging
        {
            var input = string.Format(format, args);
            if (input != _last)
            {
                _last = input;
                Logging.Write(Colors.LawnGreen, "[Bg Bot]{0}", input);
            }
        }

        private bool NeedToStopMovement
        {
            get
            {
                return CurrentHotspot.Position.Distance(Me.Location) < CurrentHotspot.stopRadius && Me.IsMoving;
            }
        }

        private static bool NeedToMoveToTarget(object context)
        {
            if (Targeting.Instance.FirstUnit == null)
            {
                return false;
            }

            if (Targeting.Instance.FirstUnit.DistanceSqr < Targeting.PullDistance * Targeting.PullDistance && Targeting.Instance.FirstUnit.InLineOfSight)
            {
                return false;
            }

            return true;
        }

        private static bool NeedToClearPoi(object context)
        {
            WoWUnit firstUnit = Targeting.Instance.FirstUnit;
            WoWUnit currentTarget = Me.CurrentTarget;

            return (BotPoi.Current.Type == PoiType.Kill
                    && firstUnit != null
                    && firstUnit.Distance < Targeting.PullDistance
                    && currentTarget == null)
                   || (currentTarget != null
                       && firstUnit != null
                       && currentTarget.Guid != firstUnit.Guid);
        }

        #endregion

        //public class LockSelector : PrioritySelector
        //{
        //    public LockSelector(params Composite[] children)
        //        : base(children)
        //    {
        //    }

        //    public override RunStatus Tick(object context)
        //    {
        //        using (StyxWoW.Memory.AcquireFrame())
        //        {
        //            return base.Tick(context);
        //        }
        //    }
        //}
    }
}


