﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms.VisualStyles;
using Styx.Common;
using Styx.CommonBot.Profiles;
using Styx.Helpers;


using Tripper.MeshMisc;
using Tripper.Navigation;
using Tripper.RecastManaged.Detour;
using Vector3 = Tripper.Tools.Math.Vector3;

namespace BgBot
{
    class BlackspotHelper
    {
        private const int QueryPolygonsCount = 8192;
        internal static void ApplyBlackspots(WowNavigator wowNav, IEnumerable<Blackspot> blackspots)
        {
            if (blackspots == null)
                return;

            wowNav.QueryFilter.SetAreaCost(AreaType.Misc7, 60f);
            foreach (Blackspot spot in blackspots)
            {
                Vector3 blackSpotPosOpenGL = NavHelper.ToNav(spot.Location);
                var extents = new Vector3(spot.Radius, spot.Height, spot.Radius);

                PolygonReference[] polys;
                Status status = wowNav.MeshQuery.QueryPolygons(blackSpotPosOpenGL, extents, wowNav.QueryFilter.InternalFilter, QueryPolygonsCount, out polys);

                if (status.Failed)
                {
                    Logging.Write("Warning: QueryPolygons failed while applying blackspot {0} with radius {1} (status: {2})", spot.Location, spot.Radius, status.Flags);
                }
                else
                {
                    foreach (PolygonReference poly in polys)
                    { 
                        wowNav.Mesh.SetPolyArea(poly, (byte)AreaType.Misc7);
                    }
                }
            }
        }
    }
}
