﻿using System.Collections.Generic;
using System.Linq;

using Styx;
using Styx.Pathing;
using Styx.WoWInternals.WoWObjects;

using Tripper.Tools.Math;

namespace BgBot
{
    class ClusterManager
    {
        private const float MAX_CLUSTER_RANGE = 40f;

        public static WoWPartyMember GetBestPartyMember(List<WoWPartyMember> allInfos)
        {
            WoWPartyMember bestMember = allInfos[0];
            int bestClusterSize = (from m in allInfos
                                   where m != bestMember && m.Location.DistanceSqr(bestMember.Location) < MAX_CLUSTER_RANGE * MAX_CLUSTER_RANGE
                                   select m).Count();

            
            foreach (var mi in allInfos)
            {
                // Don't need it!
                if (mi == bestMember)
                    continue;

                int clusterSize = (from m in allInfos
                                   where m != mi && m.Location.DistanceSqr(mi.Location) < MAX_CLUSTER_RANGE * MAX_CLUSTER_RANGE
                                   select m).Count();

                if (clusterSize > bestClusterSize)
                {
                    bestClusterSize = clusterSize;
                    bestMember = mi;
                }
            }

            return bestMember;
        }

        public static WoWPoint GetBestLocation()
        {
            var allInfos = StyxWoW.Me.GroupInfo.RaidMembers.Where(m => m.Guid != StyxWoW.Me.Guid).ToList();
            var bestMember = GetBestPartyMember(allInfos);

            var allPoints = (from m in allInfos
                             where m.Location.DistanceSqr(bestMember.Location) < MAX_CLUSTER_RANGE * MAX_CLUSTER_RANGE
                             select m.Location).ToList();

            Vector2 final = Vector2.Zero;

            // Add all the points within, so we can get an average.
            foreach (var allPoint in allPoints)
            {
                final += allPoint;
            }

            // Divide by the number of locs, so we have an average location to get to!
            final /= allPoints.Count;

            float z;
            Navigator.FindHeight(final.X, final.Y, out z);
            return new WoWPoint(final.X, final.Y, z);
        }
    }
}
