﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;

using Styx;
using Styx.Common;
using Styx.CommonBot;


using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Vector2 = Tripper.Tools.Math.Vector2;

namespace BgBot.Logic
{
    public class AlteracValley : BoxOnlyBattleground
    {
        public override PlayStyle Stance
        {
            get
            {
                var ps = BBRSettings.Instance.avplaystyle;
                return ps != PlayStyle.Random ? _randomPlaystyle : ps;
            }
        }

        public struct Node
        {
            public LandmarkControlType Default;
            public WoWPoint Location;

            public Node(WoWPoint p, LandmarkControlType d)
            {
                Default = d;
                Location = p;
            }
        }

        public static Dictionary<String, Node> AVLocations = new Dictionary<String, Node>
                                                 {
{"Coldtooth Mine", new Node(new WoWPoint(-948.542,-193.079,99.81518),LandmarkControlType.Uncontrolled)},
{"Dun Baldar", new Node(new WoWPoint(693.095,-19.197,50.65477),LandmarkControlType.Uncontrolled)},
{"Dun Baldar North Bunker", new Node(new WoWPoint(668.04,-128.176,49.76817),LandmarkControlType.AllianceControlled)},
{"Dun Baldar South Bunker", new Node(new WoWPoint(571.535,-80.543,38.05477),LandmarkControlType.AllianceControlled)},
{"East Frostwolf Tower", new Node(new WoWPoint(-1302.65,-305.886,113.8778),LandmarkControlType.HordeControlled)},
{"Frostwolf Graveyard", new Node(new WoWPoint(-1083.31,-307.402,56.94368),LandmarkControlType.HordeControlled)},
{"Frostwolf Keep", new Node(new WoWPoint(-1369.01,-225.37,91.27778),LandmarkControlType.Uncontrolled)},
{"Frostwolf Relief Hut", new Node(new WoWPoint(-1415.05,-305.986,89.65902),LandmarkControlType.HordeControlled)},
{"Iceblood Garrison", new Node(new WoWPoint(-537.807,-168.363,57.22445),LandmarkControlType.Uncontrolled)},
{"Iceblood Graveyard", new Node(new WoWPoint(-531.316,-405.584,49.76348),LandmarkControlType.HordeControlled)},
{"Iceblood Tower", new Node(new WoWPoint(-562.172,-273.554,52.63863),LandmarkControlType.HordeControlled)},
{"Icewing Bunker", new Node(new WoWPoint(217.425,-359.764,42.53041),LandmarkControlType.AllianceControlled)},
{"Icewing Cavern", new Node(new WoWPoint(276.056,61.1517,20.53205),LandmarkControlType.Uncontrolled)},
{"Irondeep Mine", new Node(new WoWPoint(801.41,-330.128,61.59412),LandmarkControlType.Uncontrolled)},
{"Snowfall Graveyard", new Node(new WoWPoint(-202.932,-112.716,78.62625),LandmarkControlType.Uncontrolled)},
{"Stonehearth Bunker", new Node(new WoWPoint(-160.17,-454.107,50.78919),LandmarkControlType.AllianceControlled)},
{"Stonehearth Graveyard", new Node(new WoWPoint(65.9825,-415.573,49.78914),LandmarkControlType.AllianceControlled)},
{"Stonehearth Outpost", new Node(new WoWPoint(1.66764,-292.675,15.68436),LandmarkControlType.Uncontrolled)},
{"Stormpike Aid Station", new Node(new WoWPoint(640.261,-18.6661,49.0409),LandmarkControlType.AllianceControlled)},
{"Stormpike Graveyard", new Node(new WoWPoint(666.927,-305.544,30.13313),LandmarkControlType.AllianceControlled)},
{"Tower Point", new Node(new WoWPoint(-760.664,-347.326,67.67035),LandmarkControlType.HordeControlled)},
{"West Frostwolf Tower", new Node(new WoWPoint(-1297.9,-266.613,91.95909),LandmarkControlType.HordeControlled)},
{"Wildpaw Cavern", new Node(new WoWPoint(-1436.1,-455.651,70.66004),LandmarkControlType.Uncontrolled)}

                                                 };



        private readonly Regex _reinforcementsRegex = new Regex(@"\d+", RegexOptions.CultureInvariant | RegexOptions.Compiled);
        public int Reinforcements
        {
            get
            {

                int dataIndex = 1;
                if (StyxWoW.Me.IsHorde)
                    dataIndex = 2;
                var val = Lua.GetReturnVal<string>(string.Format("return GetWorldStateUIInfo({0})", dataIndex), 3);

                // Yes, we use Regex here. There's no other 'simple' way to get this number. (If you find one, please, feel free to change this code)
                string value = _reinforcementsRegex.Match(val).Value;
                int ret = int.Parse(value);
                return ret;
            }
        }

        public override BattlegroundType BgType
        {
            get { return BattlegroundType.AV; }
        }

        public override string ProfileName
        {
            get
            {
                return "AlteracValley";
            }
        }


        private LandmarkControlType OurSide
        {
            get
            {
                return StyxWoW.Me.IsHorde ? LandmarkControlType.HordeControlled : LandmarkControlType.AllianceControlled;
            }
        }

        private LandmarkControlType TheirSide
        {
            get
            {
                return StyxWoW.Me.IsAlliance ? LandmarkControlType.HordeControlled : LandmarkControlType.AllianceControlled;
            }
        }

        public override PvpHotspot GetFightLocation(IEnumerable<Vector2> playerLocations, IEnumerable<PvpHotspot> hotspots)
        {

            foreach (var h in hotspots)
            {
                h.LastPlayerCount = 0;
            }

            // If the game isn't mature yet, just go to Balinda.
            if (Reinforcements > 490)
            {
                    if (StyxWoW.Me.IsHorde)
                    {
                        return hotspots.First(h => h.Name == "Balinda");
                    }
                    else
                    {
                        return hotspots.First(h => h.Name == "Galvangar");
                    }  
            }


           StyxWoW.Landmarks.Refresh();

            //Offensive 
            //Find nearest tower with that belongs to the enemy with allies nearby
            if (Stance == PlayStyle.Offensive)
            {
                //Get the nodes we care about.
                var goobers = StyxWoW.Landmarks.LandmarkList.Select(z => z.ToAlteracValleyLandmark()).Where(r => AVLocations[r.Name].Default == TheirSide && r.ControlType == TheirSide);

                AlteracValleyLandmark best = null;
                int bestCount = int.MinValue;

                //Sort through them and find the one with the least amount of allies.
                foreach (var goober in goobers)
                {
                    var count = RaidHelper.GetFriendlyUnitsNearPoint(goober.Location, 35, playerLocations);
                    if (count > bestCount)
                    {
                        best = goober;
                        bestCount = count;
                    }

                }

                if (best != null)
                {
                        return new PvpHotspot("Gonna attack " + best.Name, best.Location);
                }
            }

            //Defensive
            //Find nearest tower that has been captured with allies
            if (Stance == PlayStyle.Defensive)
            {
                //Get the nodes we care about.
                var goobers = StyxWoW.Landmarks.LandmarkList.Select(z => z.ToAlteracValleyLandmark()).Where(r => AVLocations[r.Name].Default == TheirSide && r.ControlType == LandmarkControlType.InConflict);

                AlteracValleyLandmark best = null;
                AlteracValleyLandmark player = null;
                int bestCount = int.MaxValue;
                int playerCount = 0;

                //Sort through them and find the one with the least amount of allies.
                foreach (var goober in goobers)
                {
                    var count = RaidHelper.GetFriendlyUnitsNearPoint(goober.Location, 35, playerLocations);
                    if (count < bestCount)
                    {
                        best = goober;
                        bestCount = count;
                    }

                    if (StyxWoW.Me.Location.Distance(goober.Location) < 35)
                    {
                        player = goober;
                        playerCount = count;
                    }

                }

                if (best != null)
                {
                    if (player != null)
                    {

                        //5 at current location, 3 at other
                        //(5-3) = (2 < 3) = stay at current location
                        if (playerCount - 3 < bestCount)
                        {
                            return new PvpHotspot("Staying at " + player.Name, player.Location);
                        }
                        else
                        {
                            return new PvpHotspot("Gonna defend " + best.Name, best.Location);
                        }
                    }
                    else
                    {
                        return new PvpHotspot("Gonna defend " + best.Name, best.Location);
                    }
                }

            }

            return base.GetFightLocation(playerLocations, hotspots);
        }

        public override void WeighTargetsFilter(List<Targeting.TargetPriority> units)
        {
            foreach (var targetPriority in units)
            {
                var u = targetPriority.Object.ToUnit();

                // This should take care of all bosses. (Balinda, Vandaar, Drak, etc)
                if (u.Elite && u.Combat)
                    targetPriority.Score += 300f;
            }
        }


        //o.Entry == 11949 || //Balinda
        //o.Entry == 11948) && // Vandaar
        //o.Entry == 11946 || // Drek'thar
        //o.Entry == 11947) && // Galvangar

        public override void IncludeTargetsFilter(List<WoWObject> incomingunits, HashSet<WoWObject> outgoingunits)
        {
            var hordeOnly = StyxWoW.Me.IsAlliance;
            var bosses = ObjectManager.GetObjectsOfType<WoWUnit>(false, false).
                Where(
                    o =>
                    (((o.Entry == 11949 || o.Entry == 11948) && !hordeOnly) ||
                    ((o.Entry == 11946 || o.Entry == 11947) && hordeOnly)) && o.Combat);


            // Add our specific bosses to the mix.
            foreach (var woWUnit in bosses)
            {
                outgoingunits.Add(woWUnit);
            }

            foreach (var incomingunit in incomingunits)
            {
                outgoingunits.Add(incomingunit);
            }
        }
    }
}