﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Vector2 = Tripper.Tools.Math.Vector2;



namespace BgBot
{


    
    public abstract class Battleground : IDisposable
    {
        public abstract BattlegroundType BgType { get; }

        public abstract string ProfileName { get; }

        public void ChooseRandom()
        {
            Random rng = new Random();
            var choice = rng.Next(0, 2);

            if (choice == 0)
            {
                _randomPlaystyle = PlayStyle.Defensive;
            }
            else if (choice == 1)
            {
                _randomPlaystyle = PlayStyle.Neutral;
            }
            else if (choice == 2)
            {
                _randomPlaystyle = PlayStyle.Offensive;
            }

        }

        public PlayStyle _randomPlaystyle;
        public virtual PlayStyle Stance
        {
            get
            {
                return PlayStyle.Neutral;
            }
        }
        protected Battleground()
        {
            Targeting.Instance.WeighTargetsFilter += WeighTargetsFilter;
            Targeting.Instance.IncludeTargetsFilter += IncludeTargetsFilter;
            ChooseRandom();
        }


        /// <summary>Battleground specific unit target inclusions.</summary>
        /// <param name="incomingunits">The incomingunits.</param>
        /// <param name="outgoingunits">The outgoingunits.</param>
        public virtual void IncludeTargetsFilter(List<WoWObject> incomingunits, HashSet<WoWObject> outgoingunits)
        {
            
        }



        /*(public virtual void LandMarks()
        {

            

            for (uint i = 0; i < this.NumMapLandmarks; i++)
            {
                WoWLandMark item = new WoWLandMark(StyxWoW.Memory.Read<IntPtr>(StyxWoW.Offsets.struct36_0.intptr_59, false) + ((IntPtr)(0x10 * i)));
                this.LandmarkList.Add(item);
            }
            Styx.Offsets.WoWGameObjectFields
        }*/

        /// <summary>Gets a fight location for this battleground</summary>
        /// <param name="playerLocations">The player locations.</param>
        /// <param name="hotspots">The hotspots.</param>
        /// <returns>The fight location.</returns>
        public virtual PvpHotspot GetFightLocation(IEnumerable<Vector2> playerLocations, IEnumerable<PvpHotspot> hotspots)
        {
            //return PvpProfile.Current.GetCurrentHotspot();

            //Wtf this used to return only null...
            foreach (var h in hotspots)
            {
                h.LastPlayerCount = 0;
            }
            foreach (var h in hotspots)
            {
                h.LastPlayerCount = RaidHelper.GetCountWithin(h.Box, playerLocations);
                if (h.LastPlayerCount != 0)
                    Logging.WriteDiagnostic(h.Name + " => " + h.LastPlayerCount);
            }

            PvpHotspot best = GetHighestDensity(hotspots);

            // Return the best, or the default if there is no current 'best'.
            return best ?? hotspots.First(h => h.IsDefault);
        }

        /// <summary>Battleground specific unit weighting.</summary>
        /// <param name="units">The units.</param>
        public virtual void WeighTargetsFilter(List<Targeting.TargetPriority> units)
        {
            
        }
        
        //protected virtual PvpHotspot GetLowestDensity(IEnumerable<PvpHotspot> hotspots)
        //{
        //    if (hotspots.All(h => h.LastPlayerCount == 0))
        //        return null;
        //    var ordered = hotspots.Where(h => h != null).OrderByDescending(h => h.LastPlayerCount).ToList();

        //    int bestHotspotIndex = 0;
        //    for (int i = 0; i < ordered.Count; i++)
        //    {
        //        // Skip the best hotspot 
        //        if (ordered[i] == null || i == bestHotspotIndex)
        //            continue;

        //        if (ordered[i].LastPlayerCount * 1.2 < ordered[bestHotspotIndex].LastPlayerCount)
        //        {
        //            bestHotspotIndex = i;
        //        }
        //    }
        //    return ordered[bestHotspotIndex];
        //}

        protected virtual PvpHotspot GetHighestDensity(IEnumerable<PvpHotspot> hotspots)
        {
            if (hotspots.Where(h => h != null).All(h => h.LastPlayerCount == 0))
                return null;

            var ordered = hotspots.Where(h => h != null).OrderByDescending(h => h.LastPlayerCount).ToList();
            var current = BgBot.Instance.CurrentHotspot;
            int bestHotspotIndex = 0;
            if (current != null && ordered.Contains(current))
            {
                bestHotspotIndex = Math.Max(ordered.ToList().IndexOf(current), 0);
            }
            
            for (int i = 0; i < ordered.Count; i++)
            {
                if (ordered[i] == null || i == bestHotspotIndex)
                    continue;

                if (ordered[i].LastPlayerCount * 1.2f > ordered[bestHotspotIndex].LastPlayerCount)
                {
                    // No longer the best hotspot!
                    //ordered[bestHotspotIndex].IsCurrent = false;
                    bestHotspotIndex = i;
                }
            }
            //ordered[bestHotspotIndex].IsCurrent = true;
            return ordered[bestHotspotIndex];
        }

        private bool _isDisposed;

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged
        /// resources.
        /// </summary>
        public virtual void Dispose()
        {
            if (!_isDisposed)
            {
                Targeting.Instance.WeighTargetsFilter -= WeighTargetsFilter;
                Targeting.Instance.IncludeTargetsFilter -= IncludeTargetsFilter;
                _isDisposed = true;
            }
        }

        ~Battleground()
        {
            Dispose();
        }
    }
}