﻿using System.Collections.Generic;
using System.Linq;

using Styx;
using Styx.CommonBot;
using Styx.CommonBot.POI;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

using Enumerable = System.Linq.Enumerable;

namespace BgBot.Logic
{
    /// <summary>Represents a CTF-based battleground. (Capture the flag)</summary>
    /// <remarks>Created 12/27/2010.</remarks>
    public abstract class CtfBattleground : BoxOnlyBattleground
    {
        public override PvpHotspot GetFightLocation(IEnumerable<Tripper.Tools.Math.Vector2> playerLocations, IEnumerable<PvpHotspot> hotspots)
        {
            foreach (var h in hotspots)
            {
                h.LastPlayerCount = 0;
            }
            var friendlyCarrier = FriendlyFlagCarrier;
            if (friendlyCarrier != null )
            {
                if (friendlyCarrier.IsMe)
                {
                    // We have our own flag... set our location to our own flag!
                    if (StyxWoW.Me.IsHorde)
                        return hotspots.GetHotspot("HordeFlag");
                    return hotspots.GetHotspot("AllianceFlag");
                }
                else if (Stance == PlayStyle.Defensive)
                {
                    return new PvpHotspot("Flag Carrier", friendlyCarrier);
                }
            }

            var enemyFlagCarrier = EnemyFlagCarrier;

            if (enemyFlagCarrier != null)
            {


                if (enemyFlagCarrier.DistanceSqr < 35 * 35 && enemyFlagCarrier.InLineOfSight)
                {
                    // Set the POI to the carrier, if it isn't already.
                    if (BotPoi.Current.Guid != enemyFlagCarrier.Guid)
                    {
                        // Clear the POI, and set it to the carrier.
                        BotPoi.Clear("Switching to flag carrier!");
                        BotPoi.Current = new BotPoi(enemyFlagCarrier, PoiType.Kill);
                        enemyFlagCarrier.Target();
                    }

                    // Leave this outside of the POI check, so we're always updating the location of the carrier!
                    // Create a 'custom' hotspot for the time being.
                    return new PvpHotspot("Flag Carrier", enemyFlagCarrier);
                }
                else if (Stance == PlayStyle.Offensive)
                {
                    return new PvpHotspot("Flag Carrier", enemyFlagCarrier);
                }
            }

            return base.GetFightLocation(playerLocations, hotspots);
            
            //if (StyxWoW.Me.IsHorde)
            //    return hotspots.GetHotspot("AllianceFlag");
            //return hotspots.GetHotspot("HordeFlag");
        }

        public virtual string FriendlyFlagBuff
        {
            get
            {
                if (StyxWoW.Me.IsHorde)
                    return HordeFlagBuff;
                return AllianceFlagBuff;
            }
        }

        public virtual string EnemyFlagBuff
        {
            get
            {
                if (StyxWoW.Me.IsHorde)
                    return AllianceFlagBuff;
                return HordeFlagBuff;
            }
        }

        protected virtual string HordeFlagBuff { get { return "Horde Flag"; } }

        protected virtual string AllianceFlagBuff { get { return "Alliance Flag"; } }

        public virtual WoWPlayer EnemyFlagCarrier
        {
            get
            {
                return ObjectManager.GetObjectsOfType<WoWPlayer>(true, false).FirstOrDefault(p => p.HasAura(FriendlyFlagBuff));
            }
        }

        public virtual WoWPlayer FriendlyFlagCarrier
        {
            get
            {
                // Include ourselves in this search, since we may be the actual carrier!
                return ObjectManager.GetObjectsOfType<WoWPlayer>(true, true).FirstOrDefault(p => p.HasAura(EnemyFlagBuff));
            }
        }

        public override void WeighTargetsFilter(List<Targeting.TargetPriority> units)
        {
            string flagAura = EnemyFlagBuff;

            foreach (var target in units)
            {
                // If it's not a player, ignore it.
                var p = target.Object.ToPlayer();
                if (p == null)
                    continue;

                // We want to target the flag carrier most. Only if its within range. Kthx.
                if (p.HasAura(flagAura) && p.Distance <= 40)
                    target.Score += 200f;
            }
        }
    }
}