﻿using System;
using System.Collections.Generic;
using System.Linq;
using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.CommonBot.POI;
using Styx.Helpers;

using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Vector2 = Tripper.Tools.Math.Vector2;

namespace BgBot.Logic
{
    public enum EotsLocations
    {
        BloodElfTower,
        DraeneiRuins,
        FelReaverRuins,
        MageTower,
        Flag
    }




    public class EyeOfTheStorm : Battleground
    {


        public override PlayStyle Stance
        {
            get
            {
                var ps = BBRSettings.Instance.eotsPlaystyle;
                return ps != PlayStyle.Random ? _randomPlaystyle : ps;
            }
        }


        public override void WeighTargetsFilter(List<Targeting.TargetPriority> units)
        {
            foreach (var tp in units)
            {
                if (tp.Object is WoWUnit && tp.Object.ToUnit().HasAura("Netherstorm Flag"))
                {
                    tp.Score += 200f;
                }
            }
        }

        //private readonly Dictionary<EotsLocations, MapBox> _fightBoxes = new Dictionary<EotsLocations, MapBox>
        //    {
        //        { EotsLocations.BloodElfTower, new MapBox(0.52f, 0.59f, 0.54f, 0.60f) },
        //        { EotsLocations.DraeneiRuins, new MapBox(0.53f, 0.57f, 0.32f, 0.38f) },
        //        { EotsLocations.FelReaverRuins, new MapBox(0.36f, 0.45f, 0.52f, 0.62f) },
        //        { EotsLocations.MageTower, new MapBox(0.37f, 0.46f, 0.37f, 0.46f) },
        //        { EotsLocations.Flag, new MapBox(0.46f, 0.50f, 0.46f, 0.52f) },
        //    };

        public virtual WoWPlayer EnemyFlagCarrier
        {
            get
            {
                return ObjectManager.GetObjectsOfType<WoWPlayer>(true, false).FirstOrDefault(p => p.HasAura("Netherstorm Flag") && p.IsHostile);
            }
        }

        public virtual WoWPlayer FriendlyFlagCarrier
        {
            get
            {
                // Include ourselves in this search, since we may be the actual carrier!
                return ObjectManager.GetObjectsOfType<WoWPlayer>(true, true).FirstOrDefault(p => p.HasAura("Netherstorm Flag") && p.IsFriendly);
            }
        }

        public override PvpHotspot GetFightLocation(IEnumerable<Tripper.Tools.Math.Vector2> playerLocations, IEnumerable<PvpHotspot> hotspots)
        {
            foreach (var h in hotspots)
            {
                h.LastPlayerCount = 0;
            }

            var enemyFlagCarrier = EnemyFlagCarrier;

            // lets apply a sanity check before switching targets!
            if (enemyFlagCarrier != null)
            {
                if (enemyFlagCarrier.DistanceSqr < 35 * 35 && enemyFlagCarrier.InLineOfSight)
                {
                    // Set the POI to the carrier, if it isn't already.
                    if (BotPoi.Current.Guid != enemyFlagCarrier.Guid)
                    {
                        // Clear the POI, and set it to the carrier.
                        BotPoi.Clear("Switching to flag carrier!");
                        BotPoi.Current = new BotPoi(enemyFlagCarrier, PoiType.Kill);
                        enemyFlagCarrier.Target();
                    }

                    // Leave this outside of the POI check, so we're always updating the location of the carrier!
                    // Create a 'custom' hotspot for the time being.
                    return new PvpHotspot("Flag Carrier", enemyFlagCarrier);
                }
                else if (Stance == PlayStyle.Offensive)
                {
                    return new PvpHotspot("Flag Carrier", enemyFlagCarrier);
                }
            }

            //Defensive Logic
            if (Stance == PlayStyle.Defensive && FriendlyFlagCarrier != null)
            {
                return new PvpHotspot("Flag Carrier", FriendlyFlagCarrier);
            }

            //Standard logic.
            return base.GetFightLocation(playerLocations, hotspots);

            //if (StyxWoW.Me.IsHorde)
            //    return hotspots.GetHotspot("AllianceFlag");
            //return hotspots.GetHotspot("HordeFlag");
        }





        /// <summary>Gets a count of players at a specified hotspot</summary>
        /// <remarks>Created 12/22/2010.</remarks>
        /// <param name="hotspots">The hotspots.</param>
        /// <param name="players">The players.</param>
        /// <param name="lmType">Type of the landmark to get.</param>
        /// <returns>The count.</returns>
        private PvpHotspot GetCount(IEnumerable<PvpHotspot> hotspots, IEnumerable<Vector2> players, EotsLocations lmType)
        {
            // Get it by name, if the named hotspot doesn't exist, print a messsage. (Yes, spammy, but they'll get the damned message)
            PvpHotspot hs = hotspots.GetHotspot(lmType.ToString());

            if (hs != null)
            {
                // Set the count based on the type we're requesting.
                hs.LastPlayerCount = RaidHelper.GetCountWithin(hs.Box, players);
            }

            // If the player count is 0 on a contested landmark, fucking ignore it!
            // Disable the hs == null context marker. It CAN be null.
            // ReSharper disable ConditionIsAlwaysTrueOrFalse
            if (hs == null || hs.LastPlayerCount == 0)
                // ReSharper restore ConditionIsAlwaysTrueOrFalse
                return null;
            return hs;
        }

        private bool FlagCanBeUsed()
        {
            Battlegrounds.LandMarks.Refresh();
            int alliControl = Battlegrounds.LandMarks.LandmarkList.Count(l => l.ToEyeOfTheStormLandmark().ControlType == LandmarkControlType.AllianceControlled);
            int hordeControl = Battlegrounds.LandMarks.LandmarkList.Count(l => l.ToEyeOfTheStormLandmark().ControlType == LandmarkControlType.HordeControlled);

            return (alliControl == 2 && hordeControl == 2) || (alliControl + hordeControl == 4);
        }

        public override BattlegroundType BgType
        {
            get { return BattlegroundType.EotS; }
        }

        public override string ProfileName
        {
            get
            {
                return "EyeOfTheStorm";
            }
        }

        /*public override PvpHotspot GetFightLocation(
            IEnumerable<Vector2> playerLocations, IEnumerable<PvpHotspot> hotspots)
        {
            foreach (var h in hotspots)
            {
                h.LastPlayerCount = 0;
            }

            // Yep, simplified. Just get the hotspots, and player counts within each.
            var bet = GetCount(hotspots, playerLocations, EotsLocations.BloodElfTower);
            var dr = GetCount(hotspots, playerLocations, EotsLocations.DraeneiRuins);
            var frr = GetCount(hotspots, playerLocations, EotsLocations.FelReaverRuins);
            var mt = GetCount(hotspots, playerLocations, EotsLocations.MageTower);

            List<PvpHotspot> usables = new List<PvpHotspot> { bet, dr, frr, mt };
            if(FlagCanBeUsed())
                usables.Add(GetCount(hotspots, playerLocations, EotsLocations.Flag));

            // This just gets us the best 'weighted' place to be. If they're null, it's fine. The func should take that into account, and ignore it.
            PvpHotspot best = GetHighestDensity(usables);

            // Make sure we only move if we're not nulled out here
            if (best != null)
            {
                Logging.Write("Best fight location yo:" + best);
                return best;
            }
            // If we're not opened up yet, go to the default location. Kthx
            // Keep in mind; this is just so we have *something* when the gates open. We'll still sit and wait for them to open.
            Logging.Write("Best is default!");
            return hotspots.First(h => h.IsDefault);
        }*/
    }
}
