﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using Styx;
using Styx.Common;
using Styx.Helpers;
using Styx.WoWInternals;
using Vector2 = Tripper.Tools.Math.Vector2;
using Styx.Patchables;

namespace BgBot.Logic
{

    public struct LandmarkPlaceholder
    {
        public string Name;
        public WoWPoint Location;
        public LandmarkControlType ControlType;
        public LandmarkPlaceholder(string name, WoWPoint loc, LandmarkControlType ct)
        {
            Name = name;
            Location = loc;
            ControlType = ct;
        }
    }

    public class MyHPLandmark : AreaPoiLandmark
    {


        /*public MyHPLandmark(IntPtr ptr): base(ptr)
        {
        }*/

        public MyHPLandmark(int index, LandMarkEntry data)
            : base(index, data)
        {

        }

        public LandmarkControlType ControlType
        {
            get
            {
                switch (NormalIcon)
                {
                    case 6:
                    case 0x1a:
                    case 0x10:
                    case 0x15:
                    case 0x1f:
                    case 0x24:
                        return LandmarkControlType.Uncontrolled;

                    case 9:
                    case 12:
                    case 0x11:
                    case 0x13:
                    case 0x16:
                    case 0x18:
                    case 0x1b:
                    case 0x1d:
                    case 0x20:
                    case 0x22:
                    case 0x25:
                    case 0x27:
                        return LandmarkControlType.InConflict;

                    case 11:
                    case 0x12:
                    case 0x17:
                    case 0x1c:
                    case 0x21:
                    case 0x26:
                        return LandmarkControlType.AllianceControlled;

                    case 10:
                    case 20:
                    case 0x19:
                    case 30:
                    case 0x23:
                    case 40:
                        return LandmarkControlType.HordeControlled;
                }
                return LandmarkControlType.Unknown;
            }
        }





    }

    public static class LandmarkExtensions
    {

        public static Dictionary<String, WoWPoint> Locations = new Dictionary<String, WoWPoint>
                                                 {
                                                     //BFG
                                                     {"Lighthouse",new WoWPoint(1047.119,1287.746,3.551034)},
                                                     {"Mines",new WoWPoint(1249.563,959.337,6.164304)},
                                                     {"Waterworks",new WoWPoint(974.4443,948.5432,13.17166)},
                                                     //AB
                                                     {"Blacksmith",new WoWPoint(974.9388,1044.776,-44.62589)},
                                                     {"Farm",new WoWPoint(798.022,878.2909,-55.1918)},
                                                     {"Gold Mine",new WoWPoint(1147.498,847.794,-109.6731)},
                                                     {"Lumber Mill",new WoWPoint(859.4246,1151.004,11.53672)},
                                                     {"Stables",new WoWPoint(1167.264,1201.672,-56.42229)}

                                                 };



        public static MyHPLandmark ToMyHPLandmark(this WoWLandMark lm)
        {
            return new MyHPLandmark(lm.Index,lm.Data);
        }
    }



    public abstract class HoldPointBattleground : BoxOnlyBattleground
    {
        //Offensive
        //Find node with that is currently enemy controlled with most amount of friendly players

        //Defensive
        //Priority -> Nodes that have recently changed from InControl to Contested with highest friendly player count
        //Do a check to compare number of enemy in area proportional to number of friendlies in the area
        //Sit at a friendly point with the least amount of allys
        //If we are at a node and the number of friendly players changes at a distant node
        //We should use a delta value so we don't spam run between nodes
        private List<LandmarkPlaceholder> LastTime;

        private LandmarkControlType OurSide
        {
            get
            {
                return StyxWoW.Me.IsHorde ? LandmarkControlType.HordeControlled : LandmarkControlType.AllianceControlled;
            }
        }

        private LandmarkControlType TheirSide
        {
            get
            {
                return StyxWoW.Me.IsAlliance ? LandmarkControlType.HordeControlled : LandmarkControlType.AllianceControlled;
            }
        }

        public override PvpHotspot GetFightLocation(IEnumerable<Vector2> playerLocations, IEnumerable<PvpHotspot> hotspots)
        {
            StyxWoW.Landmarks.Refresh();
            var Current = StyxWoW.Landmarks.LandmarkList.Select(r => r.ToMyHPLandmark());

            if (LastTime == null)
            {
                LastTime = Current.Select(r => new LandmarkPlaceholder(r.Name, r.Location, r.ControlType)).ToList();
            }
            else
            {
                //Offensive
                if (Stance == PlayStyle.Offensive)
                {
                    var wanted =
                        Current.Where(r => r.ControlType == TheirSide || r.ControlType == LandmarkControlType.InConflict);
                    MyHPLandmark best = null;
                    int numFriendlies = 0;


                    foreach (var x in wanted)
                    {
                        var count = RaidHelper.GetFriendlyUnitsNearPoint(x.Location, 40, playerLocations);
                        if (count > numFriendlies)
                        {
                            best = x;
                            numFriendlies = count;
                        }
                    }


                    if (best != null)
                    {
                        return new PvpHotspot("Attack " + best.Name, best.Location);
                    }

                }
                //Defensive
                else if (Stance == PlayStyle.Defensive)
                {
                    //First find all nodes that were incontrol for our side
                    MyHPLandmark best = null;
                    int numFriendlies = 0;
                    float ratio = 0; //(float)Friendlies/Enemies - More enemies (.4322231) - More allies (1.32131)
                    //Secondary - Lets go sit at a point
                    MyHPLandmark worst = null;
                    int worstFriendlies = int.MaxValue;

                    //Lets only enumerate the list one time, do both checks inside this loop
                    for (int i = 0; i < LastTime.Count(); i++)
                    {
                        var node = LastTime[i];
                        var firstOrDefault = Current.FirstOrDefault(r => r.Name == node.Name);
                        if (firstOrDefault != null)
                        {
                            if (node.ControlType == OurSide)
                            {
                                if (firstOrDefault.ControlType == LandmarkControlType.InConflict)
                                {
                                    //Limits due to objects further not being shown
                                    if (firstOrDefault.Location.Distance2D(StyxWoW.Me.Location) < 250)
                                    {
                                        var friendlies = RaidHelper.GetFriendlyUnitsNearPoint(firstOrDefault.Location,
                                                                                              40, playerLocations);
                                        var hostile = RaidHelper.GetUnFriendlyUnitsNearPoint(firstOrDefault.Location, 40);

                                        float r = (float)friendlies / hostile;
                                        BgBot.Log("{0}-{1}-{2}-{3}", node.Name, friendlies, hostile, r);
                                        //Already more then enough of our team there.
                                        if (r > 1.6)
                                            continue;


                                        //Not too big of a difference, we can help!
                                        if (r <= 1 && r >= .75)
                                        {
                                            if (best == null)
                                            {
                                                best = firstOrDefault;
                                                numFriendlies = friendlies;
                                                ratio = r;
                                            }
                                            else
                                            {
                                                if (r < ratio)
                                                {
                                                    //This one needs more help
                                                    best = firstOrDefault;
                                                    ratio = r;
                                                }
                                            }
                                        }
                                    }

                                }
                                //Check for spots we should idle at
                                else if (firstOrDefault.ControlType == OurSide)
                                {
                                    if (best == null)
                                    {

                                        var players = RaidHelper.GetFriendlyUnitsNearPoint(firstOrDefault.Location, 40,
                                                                                           playerLocations);
                                        BgBot.Log("{0}-{1}-{2}", node.Name, players, worstFriendlies);


                                        if (players <= worstFriendlies)
                                        {
                                            worst = firstOrDefault;
                                            worstFriendlies = players;
                                        }
                                    }
                                }
                            }
                            else
                            {
                                node.ControlType = firstOrDefault.ControlType;
                            }


                        }
                    }




                    if (best != null)
                    {
                        return new PvpHotspot("Defend " + best.Name, best.Location);
                    }
                    if (worst != null)
                    {
                        return new PvpHotspot("Guard " + worst.Name, worst.Location);
                    }
                }
            }


            return base.GetFightLocation(playerLocations, hotspots);
        }


    }
}
