﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Timers;
using Bots.BGBuddy.Logic.Battlegrounds;
using Styx;
using Styx.Common;
using Styx.Helpers;
using Styx.Pathing;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Tripper.MeshMisc;
using Tripper.Navigation;
using Tripper.RecastManaged.Detour;
using Action = Styx.TreeSharp.Action;

namespace BgBot.Logic
{
    public class IsleOfConquest : BoxOnlyBattleground
    {


        private IEnumerable<StrandOfTheAncientsLandmark> marks;
        private Dictionary<int, LandmarkInfo> nodes;
        private MeshNavigator meshNav;
        private Composite GateLogic;
        private bool _CheckGates;
        private Timer CheckGates;

        public override BattlegroundType BgType
        {
            get { return BattlegroundType.IoC; }
        }


        public override string ProfileName
        {
            get
            {
                return "IsleOfConquest";
            }
        }

        public IsleOfConquest()
        {
            meshNav = Navigator.NavigationProvider as MeshNavigator;

            nodes = new Dictionary<int, LandmarkInfo>();


            CheckGates = new Timer();
            CheckGates.Elapsed += CheckGates_Elapsed;
            CheckGates.Interval = 5000;
            CheckGates.Enabled = true;


            GateLogic = new Decorator(r => _CheckGates, new Action(delegate
            {

                _CheckGates = false;
                method_17();



            }));

            meshNav.Nav.OnTileLoaded += Nav_OnTileLoaded;
            meshNav.Nav.OnSubTileLoaded += Nav_OnSubTileLoaded;

            method_17();

        }

        void CheckGates_Elapsed(object sender, ElapsedEventArgs e)
        {
            _CheckGates = true;
        }
        public override void Dispose()
        {

            CheckGates.Enabled = false;
            BgBot.Log("Cleaning up");
            TreeHooks.Instance.RemoveHook("BgBot_Special_1", GateLogic);
            meshNav.Nav.OnTileLoaded -= Nav_OnTileLoaded;
            meshNav.Nav.OnSubTileLoaded -= Nav_OnSubTileLoaded;
            base.Dispose();
        }

        private static Func<LandmarkInfo, bool> func_25;
        private IEnumerable<IsleOfConquestLandmark> Statuses
        {
            get
            {
                return Battlegrounds.LandMarks.LandmarkList.Select(l => l.ToIsleOfConquestLandmark());

            }
        }


        private void method_17()
        {
            /*Class590.Class616 @class = new Class590.Class616();
            Battlegrounds.LandMarks.Refresh();
            Class590.Class616 arg_3F_0 = @class;
            IEnumerable<WoWLandMark> arg_3A_0 = Battlegrounds.LandMarks.LandmarkList;
            if (Class590.func_25 == null)
            {
                Class590.func_25 = new Func<WoWLandMark, StrandOfTheAncientsLandmark>(Class590.smethod_72);
            }
            arg_3F_0.ienumerable_0 = arg_3A_0.Select(Class590.func_25);*/
            BgBot.Log("Checking gate statues");
            Battlegrounds.LandMarks.Refresh();

            bool flag = false;
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.AppendLine();
            foreach (IsleOfConquestLandmark current in Statuses)
            {

                if (!nodes.ContainsKey((int)current.LandmarkType))
                {
                    nodes.Add((int)current.LandmarkType, new LandmarkInfo((int)current.LandmarkType, current.ControlType, new Bots.BGBuddy.MapBox(), null));
                    if ((current.LandmarkType & IsleOfConquestLandmarkType.Gates) != IsleOfConquestLandmarkType.Unknown)
                    {
                        flag = true;
                        stringBuilder.AppendLine(string.Format("{0} has been added", current.LandmarkType));
                    }
                }
                else
                {
                    LandmarkControlType control = nodes[(int)current.LandmarkType].Control;
                    LandmarkControlType controlType = current.ControlType;
                    if (control != controlType && controlType != LandmarkControlType.Unknown && controlType != LandmarkControlType.InConflict)
                    {
                        flag = true;
                        stringBuilder.AppendLine(string.Format("State of {0} changed. Old Value: {1} New Value: {2}", current.LandmarkType, control, controlType));
                    }
                    nodes[(int)current.LandmarkType] = new LandmarkInfo((int)current.LandmarkType, current.ControlType, new Bots.BGBuddy.MapBox(), null);
                }
                //nodes[(int)current.LandmarkType].Process();

            }
            if (flag)
            {
                UnloadAllTiles(stringBuilder.ToString());
            }
            nodes.RemoveAll(method_0);
        }

        private sealed class Class615
        {
            //public Class616 class616_0;
            public LandmarkInfo landmarkInfo_0;
            public bool method_0(IsleOfConquestLandmark strandOfTheAncientsLandmark_0)
            {
                return strandOfTheAncientsLandmark_0.LandmarkType == (IsleOfConquestLandmarkType)this.landmarkInfo_0.Type;
            }
        }
        public bool method_0(LandmarkInfo landmarkInfo_0)
        {
            Class615 @class = new Class615();
            //@class.class616_0 = this;
            @class.landmarkInfo_0 = landmarkInfo_0;
            bool result;
            result = !Statuses.Any(new Func<IsleOfConquestLandmark, bool>(@class.method_0));
            return result;
        }

        private void UnloadAllTiles(string status)
        {
            meshNav.Nav.UnloadAllTiles();
            BgBot.Log("Unloading all tiles. Reason(s): {0}", status);
            Navigator.Clear();
        }

        void Nav_OnSubTileLoaded(object sender, Tripper.Navigation.TileLoadedEventArgs e)
        {

            MeshTile tileAt = ((WowNavigator)sender).Mesh.GetTileAt(e.Tile.X, e.Tile.Y);
            if (tileAt != null)
            {
                UpdateGates(tileAt);
            }

        }

        void Nav_OnTileLoaded(object sender, Tripper.Navigation.TileLoadedEventArgs e)
        {
            MeshTile tileAt = ((WowNavigator)sender).Mesh.GetTileAt(e.Tile.X, e.Tile.Y);
            if (tileAt != null)
            {
                UpdateGates(tileAt);
            }
        }

        private void UpdateGates(MeshTile meshTile_0)
        {
            bool isHorde = StyxWoW.Me.IsHorde;
            bool flag2 = StyxWoW.Me.HasAura("Teleport");

            var source = Statuses.Where(l => l.ControlType == LandmarkControlType.Destroyed);
            foreach (Poly poly in meshTile_0.GetAllPolys())
            {



                switch ((AreaType)poly.Area)
                {
                    case AreaType.AlliancePortal:
                        if (!isHorde || flag2)
                        {
                            poly.Flags = (ushort) AbilityFlags.Run;
                        }
                        else
                        {
                            poly.Flags = (ushort)AbilityFlags.Unwalkable;
                        }
                        BgBot.Log((poly.Flags == (ushort)AbilityFlags.Unwalkable) ? "Disabled alliance teleport" : "Enabled alliance teleport");
                        break;

                    case AreaType.HordePortal:
                        if (isHorde || flag2)
                        {
                            poly.Flags = (ushort) AbilityFlags.Run;
                        }
                        else
                        {
                            poly.Flags = (ushort) AbilityFlags.Unwalkable;
                        }
                        BgBot.Log((poly.Flags == 16) ? "Disabled horde teleport" : "Enabled horde teleport");
                        break;
                }

                switch (poly.Area)
                {
                    case 20:
                        poly.Flags = source.Any(l => l.LandmarkType == IsleOfConquestLandmarkType.HordeGateWest) ? (ushort)AbilityFlags.Run : (ushort)AbilityFlags.Unwalkable;
                        BgBot.Log((poly.Flags == 16) ? "Disabled horde west gate" : "Enabled horde west gate");
                        break;

                    case 21:
                        poly.Flags = source.Any(l => l.LandmarkType == IsleOfConquestLandmarkType.HordeGateFront) ? (ushort)AbilityFlags.Run : (ushort)AbilityFlags.Unwalkable;
                        BgBot.Log((poly.Flags == 16) ? "Disabled horde front gate" : "Enabled horde front gate");
                        break;

                    case 22:
                        poly.Flags = source.Any(l => l.LandmarkType == IsleOfConquestLandmarkType.HordeGateEast) ? (ushort)AbilityFlags.Run : (ushort)AbilityFlags.Unwalkable;
                        BgBot.Log((poly.Flags == 16) ? "Disabled horde east gate" : "Enabled horde east gate");
                        break;

                    case 23:
                        poly.Flags = source.Any(l => l.LandmarkType == IsleOfConquestLandmarkType.AllianceGateFront) ? (ushort)AbilityFlags.Run : (ushort)AbilityFlags.Unwalkable;
                        BgBot.Log((poly.Flags == 16) ? "Disabled alliance front gate" : "Enabled alliance front gate");
                        break;

                    case 24:
                        poly.Flags = source.Any(l => l.LandmarkType == IsleOfConquestLandmarkType.AllianceGateWest) ? (ushort)AbilityFlags.Run : (ushort)AbilityFlags.Unwalkable;
                        BgBot.Log((poly.Flags == 16) ? "Disabled alliance west gate" : "Enabled alliance west gate");
                        break;

                    case 25:
                        poly.Flags = source.Any(l => l.LandmarkType == IsleOfConquestLandmarkType.AllianceGateEast) ? (ushort)AbilityFlags.Run : (ushort)AbilityFlags.Unwalkable; ;
                        BgBot.Log((poly.Flags == 16) ? "Disabled alliance east gate" : "Enabled alliance east gate");
                        break;
                }
            }
        }






    }
}