﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Styx.CommonBot;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Tripper.Tools.Math;

namespace BgBot.Logic
{
    class TempleOfKotmogu : BoxOnlyBattleground
    {
        public override BattlegroundType BgType
        {
            get { return BattlegroundType.ToK; }
        }



        public override PlayStyle Stance
        {
            get
            {
                var ps = BBRSettings.Instance.tokPlaystyle;
                return ps != PlayStyle.Random ? _randomPlaystyle : ps;
            }
        }

        public override string ProfileName
        {
            get
            {
                return "TempleOfKotmogu";
            }
        }



        public WoWUnit FriendlyCarrier
        {
            get
            {
                return
                    ObjectManager.GetObjectsOfType<WoWUnit>(true,true)
                                 .Where(r => r.IsFriendly && r.HasAura("Orb of Power"))
                                 .OrderBy(r => r.MaxHealth)
                                 .FirstOrDefault();
            }
        }

        public WoWUnit EnemyCarrier
        {
            get
            {
                return
                    ObjectManager.GetObjectsOfType<WoWUnit>(true,true)
                                 .Where(r => !r.IsFriendly && r.HasAura("Orb of Power"))
                                 .OrderByDescending(r => r.MaxHealth)
                                 .FirstOrDefault();
            }
        }

        public override PvpHotspot GetFightLocation(IEnumerable<Vector2> playerLocations,IEnumerable<PvpHotspot> hotspots)
        {

            //Offensive, set lowest hp orb holder as hotspot.
            if (Stance == PlayStyle.Offensive && EnemyCarrier != null)
            {
                return new PvpHotspot("Orb Holder", EnemyCarrier);
            }

            if (Stance == PlayStyle.Defensive && FriendlyCarrier != null)
            {
                return new PvpHotspot("Orb Holder", FriendlyCarrier);
            }

            return base.GetFightLocation(playerLocations, hotspots);
        }

        public override void WeighTargetsFilter(List<Targeting.TargetPriority> units)
        {
            

            foreach (var target in units)
            {
                // If it's not a player, ignore it.
                var p = target.Object.ToPlayer();
                if (p == null)
                    continue;

                //Target enemies who have the orb of power cause they take more damage.
                if (p.HasAura("Orb of Power") && p.Distance <= 40)
                    target.Score += 400f;

                if (p.Distance <= 40 && p.GotTarget && FriendlyCarrier != null && p.CurrentTarget == FriendlyCarrier)
                    target.Score += 200f;

            }
        }
    }
}
