﻿using System;
using System.Globalization;
using System.Xml.Linq;
using Styx;
using Styx.Common;
using Styx.CommonBot.AreaManagement;
using Styx.Helpers;
using Styx.Pathing;
using Styx.WoWInternals.WoWObjects;

namespace BgBot
{
    public class PvpHotspot : Hotspot
    {
        public PvpHotspot(string name, WoWPoint location): base(location.X, location.Y, location.Z)
        {
            Name = name;
            stopRadius = 0;
            moveRadius = 15;
        }


        public PvpHotspot(string name, WoWObject obj, float sR = 6,float mR = 25)
            : base(obj.X, obj.Y, obj.Z)
        {
            Name = name;
            Target = obj;
            stopRadius = sR;
            moveRadius = mR;
        }

        public float stopRadius;
        public float moveRadius;

        public WoWObject Target;
        public new WoWPoint Position
        {
            get
            {
                return Target != null ? Target.Location : base.Position;
            }
            set { base.Position = value; }
        }

        

        public PvpHotspot(XElement xml)
        {
            //Box = MapBox.FromXml(xml);
            if (xml.Attribute("tlx") != null && xml.Attribute("tly") != null && xml.Attribute("brx") != null && xml.Attribute("bry") != null)
                Box = MapBox.FromXml(xml);

            float x, y, z;
            x = Convert.ToSingle(xml.Attribute("x").Value, CultureInfo.InvariantCulture);
            y = Convert.ToSingle(xml.Attribute("y").Value, CultureInfo.InvariantCulture);
            z = Convert.ToSingle(xml.Attribute("z").Value, CultureInfo.InvariantCulture);


            if (xml.Attribute("side") != null)
            {
                IsHorde = xml.Attribute("side").Value == "1";
                IsAlliance = xml.Attribute("side").Value == "0";
            }

            Name = xml.Attribute("name").Value;

            Position = new WoWPoint(x, y, z);

            if (xml.Attribute("default") != null)
                IsDefault = Convert.ToBoolean(xml.Attribute("default").Value) ;
        }

        public MapBox Box { get; set; }

        public bool IsNeutral
        {
            get
            {
                return !IsHorde && !IsAlliance;
            }
        }

        public bool IsAlliance { get; set; }
        public bool IsHorde { get; set; }

        internal int LastPlayerCount { get; set; }

        public string Name { get; set; }

        public bool IsDefault { get; set; }

        public override string ToString()
        {
            return string.Format("{3}, {0}, IsHorde: {1}, LastPlayerCount: {2}, IsDefault: {4}", base.ToString(), IsHorde, LastPlayerCount, Name, IsDefault);
        }

        //private DateTime _timeSetToCurrent = DateTime.MinValue;
        //private bool _current;
        //public bool IsCurrent
        //{
        //    get { return _current; }
        //    set
        //    {
        //        if (value && !_current)
        //        {
        //            _timeSetToCurrent = DateTime.Now;
        //        }
        //        if (!value && _current)
        //        {
        //            _timeSetToCurrent = DateTime.MinValue;
        //        }
        //        _current = value;
        //    }
        //}

        //public TimeSpan TimeBeforeBlacklist
        //{
        //    get
        //    {
        //        if (_timeSetToCurrent == DateTime.MinValue)
        //            return TimeSpan.FromMinutes(3);
                
        //        return DateTime.Now - _timeSetToCurrent.AddMinutes(3);
        //    }
        //}

        public WoWPoint RandomizedLocation { get; private set; }
        public void RandomizeLocation()
        {
            Logging.Write("Randomizing {0}'s actual location.", Name);
            
            Random rand = new Random();
            for (; ; )
            {
                var degrees = rand.Next(0, 360);
                var distance = rand.Next(0, 20);
                var endLocation = Position.RayCast(WoWMathHelper.DegreesToRadians(degrees), distance);
                float z;
                if (Navigator.FindHeight(endLocation.X, endLocation.Y, out z) && Math.Abs(endLocation.Z - z) <= 10)
                {
                    endLocation.Z = z;
                    Logging.Write(string.Format("New location: {0} at {1}deg, {2}yds.", endLocation, degrees, distance));
                    RandomizedLocation = endLocation;
                    break;
                }
            }
        }
    }
}