﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Styx;
using Styx.Helpers;
using Styx.Patchables;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

using Tripper.Tools.Math;

namespace BgBot
{
    public class RaidHelper
    {
        public static bool SetNewLeader()
        {
            if (StyxWoW.Me.IsGroupLeader)
            {
                int max = StyxWoW.Me.GroupInfo.NumRaidMembers;
                Random rand = new Random(Environment.TickCount);
                int leaderIndex = rand.Next(0, max);

                Lua.DoString("PromoteToLeader(\"" + Lua.Escape(StyxWoW.Me.RaidMembers[leaderIndex].Name) + "\")");
                return true;
            }
            return false;
        }
        public Vector2 GetMapLocation(Vector2 areaA, Vector2 areaB, Vector2 worldPoint)
        {
            Vector2 areaPos = new Vector2(0, 0);
            areaPos.X = (worldPoint.Y - areaA.Y) / (areaB.Y - areaA.Y);
            areaPos.Y = (worldPoint.X - areaA.X) / (areaB.X - areaA.X);

            return areaPos;
        }

        /// <summary>Gets a count of 'positions' within a given box. (This is for determining which 'box' has the highest concentration of players)</summary>
        /// <remarks>Created 12/19/2010.</remarks>
        /// <param name="topLeft">The top left.</param>
        /// <param name="bottomRight">The bottom right.</param>
        /// <param name="playerPositions">The player positions.</param>
        /// <returns>The count within.</returns>
        public static int GetCountWithin(Vector2 topLeft, Vector2 bottomRight, IEnumerable<Vector2> playerPositions)
        {
            //Logging.Write("Getting players within " + topLeft + " - " + bottomRight);
            //Logging.Write(playerPositions.Count() + " players to check.");
            var bb = BoundingBox2.CreateFromPoints(new[] { topLeft, bottomRight });
            var count = playerPositions.Count(bb.Contains);
            //Logging.Write("Found " + count + " here.");
            return count;
        }

        public static WoWObject GetMostPopulatedObject(IEnumerable<WoWObject> units, int Radius, IEnumerable<Vector2> playerPositions)
        {

            //We use wowpoints round these parts partner!
            var positions = playerPositions.Select(r => new WoWPoint(r.X, r.Y, 0));
            var bestCount = 0;
            WoWObject bestUnit = null;

            foreach (var unit in units)
            {
                int count = positions.Count(point => point.Distance2D(unit.Location) <= Radius);

                if (count > bestCount)
                {
                    bestUnit = unit;
                    bestCount = count;
                }

            }


            return bestUnit;
        }

        public static int GetFriendlyUnitsNearPoint(WoWPoint spot, int raidus, IEnumerable<Vector2> playerPositions)
        {
            //We use wowpoints round these parts partner!
            var positions = playerPositions.Select(r => new WoWPoint(r.X, r.Y, 0));

            int count = positions.Count(point => point.Distance2D(spot) <= raidus);


            return count;
        }

        public static int GetUnFriendlyUnitsNearPoint(WoWPoint spot, int raidus)
        {

            return
                ObjectManager.GetObjectsOfType<WoWPlayer>(true, false)
                             .Count(r => r.Location.Distance2D(spot) <= raidus && r.IsHostile);
        }

        public static WoWObject GetLeastPopulatedObject(IEnumerable<WoWObject> units, int Radius, IEnumerable<Vector2> playerPositions)
        {

            //We use wowpoints round these parts partner!
            var positions = playerPositions.Select(r => new WoWPoint(r.X, r.Y, 0));
            var bestCount = int.MaxValue;
            WoWObject bestUnit = null;

            foreach (var unit in units)
            {
                int count = positions.Count(point => point.Distance2D(unit.Location) <= Radius);

                if (count < bestCount)
                {
                    bestUnit = unit;
                    bestCount = count;
                }

            }


            return bestUnit;
        }

        internal static int GetCountWithin(MapBox box, IEnumerable<Vector2> playerPositions)
        {
            return GetCountWithin(box.TopLeft, box.BottomRight, playerPositions);
        }
    }
}
